using System;
using System.Runtime.InteropServices;
using System.Collections;

using SldWorks;
using SWPublished;


namespace SwCSharpAddin1
{
	/// <summary>
	/// Summary description for SwCSharpAddin1.
	/// </summary>
	[Guid("51413294-A125-4489-B7A3-D7F3C9656006")]
	public class SwCSharpAddin1 : SWPublished.ISwAddin
	{
		#region Local Variables
		ISldWorks iSwApp;
		SldWorks.SldWorks SwEventPtr;
		Hashtable openDocs;
		int addinID;
		int toolbarID;
		BitmapHandler bmp;
		UserPMPage ppage;
		#endregion

		#region ISwAddin Implementation
		public SwCSharpAddin1()
		{
		}

		public bool ConnectToSW(object ThisSW, int cookie)
		{
			iSwApp = (ISldWorks)ThisSW;
			SwEventPtr = (SldWorks.SldWorks)iSwApp;
			addinID = cookie;
			
			//Setup callbacks
			iSwApp.SetAddinCallbackInfo(0, this, addinID);

			//Objects used in initialization of Addin
			bmp = new BitmapHandler();
			openDocs = new Hashtable();
			AddMenus();
			AddToolbar();
			AddPMP();
			AttachEventHandlers();

			return true;
		}

		public bool DisconnectFromSW()
		{
			RemoveMenus();
			RemoveToolbar();
			RemovePMP();
						DetachEventHandlers();
			bmp.Dispose();

			iSwApp = null;
			SwEventPtr = null;
			//The addin _must_ call GC.Collect() here in order to retrieve all managed code pointers 
			GC.Collect();
			return true;
		}
		#endregion

		#region UI Methods
		public bool AddMenus()
		{
			int docType = -1;
			int pos;

			pos = 3;
			docType = (int)SwConst.swDocumentTypes_e.swDocNONE;
			iSwApp.AddMenu(docType, "SwCSharpAddin1", pos);
			iSwApp.AddMenuItem2(docType, addinID, "Create Cube@SwCSharpAddin1", -1, "MenuCallback", "MenuEnable", "Starts the notepad application");
			iSwApp.AddMenuItem2(docType, addinID, "Show PMP@SwCSharpAddin1", -1, "ShowPMP", "PMPEnable", "Displays the User PropertyManagerPage");

		   pos = 6;
			if (iSwApp.ActiveDoc != null)
			{
				IModelDoc2 modDoc = (IModelDoc2)iSwApp.ActiveDoc;
				IModelView modView = (IModelView)modDoc.GetFirstModelView();
				if (modView.FrameState != (int)SwConst.swWindowState_e.swWindowMaximized)
					pos = 5;
			}

			docType = (int)SwConst.swDocumentTypes_e.swDocPART;
			iSwApp.AddMenu(docType, "SwCSharpAddin1", pos);
			iSwApp.AddMenuItem2(docType, addinID, "Create Cube@SwCSharpAddin1", -1, "MenuCallback", "MenuEnable", "Starts the notepad application");
			iSwApp.AddMenuItem2(docType, addinID, "Show PMP@SwCSharpAddin1", -1, "ShowPMP", "PMPEnable", "Displays the User PropertyManagerPage");

			docType = (int)SwConst.swDocumentTypes_e.swDocASSEMBLY;
			iSwApp.AddMenu(docType, "SwCSharpAddin1", pos);
			iSwApp.AddMenuItem2(docType, addinID, "Create Cube@SwCSharpAddin1", -1, "MenuCallback", "MenuEnable", "Starts the notepad application");
			iSwApp.AddMenuItem2(docType, addinID, "Show PMP@SwCSharpAddin1", -1, "ShowPMP", "PMPEnable", "Displays the User PropertyManagerPage");

			docType = (int)SwConst.swDocumentTypes_e.swDocDRAWING;
			iSwApp.AddMenu(docType, "SwCSharpAddin1", pos);
			iSwApp.AddMenuItem2(docType, addinID, "Create Cube@SwCSharpAddin1", -1, "MenuCallback", "MenuEnable", "Starts the notepad application");
			iSwApp.AddMenuItem2(docType, addinID, "Show PMP@SwCSharpAddin1", -1, "ShowPMP", "PMPEnable", "Displays the User PropertyManagerPage");
			return true;
		}

		public bool RemoveMenus()
		{
			int docType = -1;
			bool success = false;
			
			docType = (int)SwConst.swDocumentTypes_e.swDocNONE;
			success = iSwApp.RemoveMenu(docType, "SwCSharpAddin1", "");

			docType = (int)SwConst.swDocumentTypes_e.swDocPART;
			success = iSwApp.RemoveMenu(docType, "SwCSharpAddin1", "");

			docType = (int)SwConst.swDocumentTypes_e.swDocASSEMBLY;
			success = iSwApp.RemoveMenu(docType, "SwCSharpAddin1", "");

			docType = (int)SwConst.swDocumentTypes_e.swDocDRAWING;
			success = iSwApp.RemoveMenu(docType, "SwCSharpAddin1", "");
		
			return true;
		}

		public bool AddToolbar()
		{
			string smallImagePath = bmp.CreateFileFromResourceBitmap("SwCSharpAddin1.ToolbarSmall.bmp");
			string largeImagePath = bmp.CreateFileFromResourceBitmap("SwCSharpAddin1.ToolbarLarge.bmp");	
			int docType =(int)SwConst.swDocTemplateTypes_e.swDocTemplateTypePART | 
						 (int)SwConst.swDocTemplateTypes_e.swDocTemplateTypeASSEMBLY |
						 (int)SwConst.swDocTemplateTypes_e.swDocTemplateTypeDRAWING;

			toolbarID = iSwApp.AddToolbar4(addinID, "SwCSharpAddin1", smallImagePath, largeImagePath, 0, docType);
			bool success = false;
			success = iSwApp.AddToolbarCommand2(addinID, toolbarID, 0, "ToolbarCallback0", "ToolbarEnable0", "TIP", "HINT");
			success = iSwApp.AddToolbarCommand2(addinID, toolbarID, 1, "ToolbarCallback1", "ToolbarEnable1", "TIP", "HINT");
			success = iSwApp.AddToolbarCommand2(addinID, toolbarID, 2, "ToolbarCallback2", "ToolbarEnable2", "TIP", "HINT");
			return true;
		}

		public bool RemoveToolbar()
		{
			iSwApp.RemoveToolbar2(addinID, toolbarID);
			return true;
		}

		public bool AddPMP()
		{
			ppage = new UserPMPage(this);
			return true;
		}

		public bool RemovePMP()
		{
			ppage = null;
			return true;
		}
		#endregion

		#region Event Methods
		public bool AttachEventHandlers()
		{
			AttachSwEvents();
            //Listen for events on all currently open docs
            ModelDoc2 modDoc;
            modDoc = (ModelDoc2)iSwApp.GetFirstDocument();
            while (modDoc != null)
			{
                if (!openDocs.Contains(modDoc)) 
				{
					AttachModelDocEventHandler(modDoc);
				}
				modDoc = (ModelDoc2)modDoc.GetNext();
			}
			return true;
		}

		private bool AttachSwEvents()
		{
			try
			{
				SwEventPtr.ActiveDocChangeNotify += new DSldWorksEvents_ActiveDocChangeNotifyEventHandler(OnDocChange);
				SwEventPtr.DocumentLoadNotify += new DSldWorksEvents_DocumentLoadNotifyEventHandler(OnDocLoad);
				SwEventPtr.FileNewNotify2 += new DSldWorksEvents_FileNewNotify2EventHandler(OnFileNew);
				SwEventPtr.ActiveModelDocChangeNotify += new DSldWorksEvents_ActiveModelDocChangeNotifyEventHandler(OnModelChange);
				return true;
			}
			catch (Exception e)
			{
				Console.WriteLine(e.Message);
				return false;
			}
		}

		private bool DetachSwEvents()
		{
			try
			{
				SwEventPtr.ActiveDocChangeNotify -= new DSldWorksEvents_ActiveDocChangeNotifyEventHandler(OnDocChange);
				SwEventPtr.DocumentLoadNotify -= new DSldWorksEvents_DocumentLoadNotifyEventHandler(OnDocLoad);
				SwEventPtr.FileNewNotify2 -= new DSldWorksEvents_FileNewNotify2EventHandler(OnFileNew);
				SwEventPtr.ActiveModelDocChangeNotify -= new DSldWorksEvents_ActiveModelDocChangeNotifyEventHandler(OnModelChange);
				return true;
			}
			catch (Exception e)
			{
				Console.WriteLine(e.Message);
				return false;
			}

		}

		public bool AttachModelDocEventHandler(ModelDoc2 modDoc)
		{
			if (modDoc == null)
                return false;
            
            DocumentEventHandler docHandler = null;

			if (!openDocs.Contains(modDoc)) 
			{
				switch(modDoc.GetType())
				{
					case (int)SwConst.swDocumentTypes_e.swDocPART:
					{
						docHandler = new PartEventHandler(modDoc, this);
						break;
					}
					case (int)SwConst.swDocumentTypes_e.swDocASSEMBLY:
					{
						docHandler = new AssemblyEventHandler(modDoc, this);
						break;
					}
					case (int)SwConst.swDocumentTypes_e.swDocDRAWING:
					{
						docHandler = new DrawingEventHandler(modDoc, this);
						break;
					}
					default:
					{
						return false; //Unsupported document type
					}
				}
				docHandler.AttachEventHandlers();
				openDocs.Add(modDoc, docHandler);
			}
			return true;
		}

		public bool DetachModelEventHandler(ModelDoc2 modDoc)
		{
			DocumentEventHandler docHandler;
			docHandler = (DocumentEventHandler)openDocs[modDoc];
			openDocs.Remove(modDoc);
			modDoc = null;
			docHandler = null;
			return true;
		}

		public bool DetachEventHandlers()
		{
			DetachSwEvents();

            //Close events on all currently open docs
            DocumentEventHandler docHandler;
            int numKeys = openDocs.Count;
			object[] keys = new Object[numKeys];

            //Remove all document event handlers
            openDocs.Keys.CopyTo(keys, 0);
			foreach(ModelDoc2 key in keys)
			{
				docHandler = (DocumentEventHandler)openDocs[key];
				docHandler.DetachEventHandlers(); //This also removes the pair from the hash
				docHandler = null;
			}
			return true;
		}
		#endregion

		#region Event Handlers
		//Events
		public int OnDocChange()
		{
			return 0;
		}

		public int OnDocLoad(string docTitle, string docPath)
		{
			ModelDoc2 modDoc = (ModelDoc2)iSwApp.GetFirstDocument();
			while(modDoc != null)
			{
				if(modDoc.GetTitle() == docTitle) 
				{
					if(!openDocs.Contains(modDoc)) 
					{
						AttachModelDocEventHandler(modDoc);
					}
				}
				modDoc = (ModelDoc2)modDoc.GetNext();
			}
			return 0;
		}

		public int OnFileNew(object newDoc, int docType, string templateName)
		{
			return 0;
		}

		public int OnModelChange()
		{
			return 0;
		}

#endregion
		
		#region Utility Methods
		//Utility Methods
		public Object GetSldWorks()
		{
			return iSwApp;
		}
		#endregion

		#region UI Callbacks
		public void MenuCallback()
		{
			//make sure we have a part open
			string partTemplate = iSwApp.GetUserPreferenceStringValue((int)SwConst.swUserPreferenceStringValue_e.swDefaultTemplatePart);
			IModelDoc2 modDoc = (IModelDoc2)iSwApp.NewDocument(partTemplate, (int)SwConst.swDwgPaperSizes_e.swDwgPaperA2size, 0.0, 0.0);

			modDoc.InsertSketch2(true);
			modDoc.SketchRectangle(0, 0, 0, .1, .1, .1, false);
			//Extrude the sketch
			IFeatureManager featMan = modDoc.FeatureManager;
			featMan.FeatureExtrusion(true, 
				false, false,
				(int)SwConst.swEndConditions_e.swEndCondBlind, (int)SwConst.swEndConditions_e.swEndCondBlind,
				0.1, 0.0, 
				false, false,
				false, false,
				0.0, 0.0,
				false, false,
				false, false,
				true,
				false, false);
		}

		public int MenuEnable()
		{
			return 1;
		}

		public void ShowPMP()
		{
			if (ppage != null)
				ppage.Show();
		}

		public int PMPEnable()
		{
			if (iSwApp.ActiveDoc != null)
				return 1;
			else
				return 0;
		}

		//Toolbar Callbacks

		public void ToolbarCallback0()
		{

		}

		public int ToolbarEnable0()
		{
			return 1;
		}

		public void ToolbarCallback1()
		{

		}

		public int ToolbarEnable1()
		{
			return 1;
		}

		public void ToolbarCallback2()
		{

		}

		public int ToolbarEnable2()
		{
			return 1;
		}
		#endregion
	}

}
